# --------------------------------------------------------------- Imports ---------------------------------------------------------------- #

# System
from typing import Optional, List, Union
from datetime import datetime
import json

# Pip
from ksimpleapi import Api

# Local
from .models import Sport, Game

# ---------------------------------------------------------------------------------------------------------------------------------------- #



# ----------------------------------------------------------- class: 365Scores ----------------------------------------------------------- #

class Scores(Api):

    # -------------------------------------------------------- Public methods -------------------------------------------------------- #

    def get_games(
        self,
        sports: Optional[Union[List[Union[Sport, int]], Union[Sport, int]]] = None,
        competition_ids: Optional[Union[List[int], int]] = None,
        start_date: Optional[Union[int, datetime, str]] = None, # 03/01/2021
        end_date: Optional[Union[int, str]] = None,   # 03/01/2021
        only_major_games: bool = True,
        only_live_games: bool = False,
        included_status_groups: List[int] = [1, 2, 3, 4],
        include_cancelled: bool = False,
        include_postponed: bool = False
    ) -> Optional[List[Game]]:
        if sports and type(sports) != list:
            sports = [sports]

        try:
            games = []

            for game_json in self._get(
                'https://webws.365scores.com/web/games/allscores',
                params={
                    'appTypeId': 5,
                    'langId': 1,
                    'startDate': self.__normalized_date(start_date),
                    'endDate': self.__normalized_date(end_date),
                    'onlyMajorGames': 'true' if only_major_games else None,
                    'onlyLiveGames': 'true' if only_live_games else None,
                    'sports': ','.join([sport.value if type(sport) == Sport else sport for sport in sports]) if sports else None,
                    'competitionIds': ','.join(competition_ids) if competition_ids else None
                },
            ).json()['games']:
                try:
                    game = Game(game_json)

                    if (
                        (not included_status_groups or game.status_group in included_status_groups) 
                        and
                        (include_cancelled or not game.cancelled)
                        and
                        (include_postponed or not game.postponed)
                    ):
                        games.append(game)
                except Exception as e:
                    if self.debug:
                        print(e, json.dumps(game_json, indent=4))

            return games
        except Exception as e:
            if self.debug:
                print(e)

            return None


    # ------------------------------------------------------- Private methods -------------------------------------------------------- #

    def __normalized_date(
        self,
        date: Optional[Union[int, datetime, str]]
    ) -> Optional[str]:
        if not date:
            return None

        if type(date) == int:
            date = datetime.fromtimestamp(date)

        if type(date) == datetime:
            date = '{}/{}/{}'.format(str(date.day).zfill(2), str(date.month).zfill(2), date.year())

        return date


# ---------------------------------------------------------------------------------------------------------------------------------------- #


# --------------------------------------------------------------- Imports ---------------------------------------------------------------- #

# System
import datetime

# Pip
from jsoncodable import JSONCodable

# Local
from .enums import Sport
from .competitor import Competitor

# ---------------------------------------------------------------------------------------------------------------------------------------- #



# ------------------------------------------------------------- class: Game -------------------------------------------------------------- #

class Game(JSONCodable):

    # ------------------------------------------------------------- Init ------------------------------------------------------------- #

    def __init__(
        self,
        d: dict
    ):
        self.id = d['id']
        self.sport_id = d['sportId']
        self.sport = Sport(self.sport_id)
        self.competition_id = d['competitionId']
        self.competition_display_name = d['competitionDisplayName'] if 'competitionDisplayName' in d else None
        self.status_group = d['statusGroup']
        self.status_text = d['statusText']
        self.short_status_text = d['shortStatusText']
        self.start_time_str = d['startTime']
        self.start_time_date = datetime.datetime.strptime(self.start_time_str.split('+')[0], '%Y-%m-%dT%H:%M:%S')
        self.start_time_ts = self.start_time_date.timestamp()
        self.just_ended = d['justEnded']
        self.postponed = self.status_group == 4 and self.status_text.lower() == 'postponed'
        self.cancelled = self.status_group == 4 and not self.postponed and self.status_text.lower() == 'cancelled'
        self.game_time = d['gameTime']

        self.home_competitor = Competitor(d['homeCompetitor'])
        self.away_competitor = Competitor(d['awayCompetitor'])

        self.score = (self.home_competitor.score, self.away_competitor.score)
        self.aggregated_score = (self.home_competitor.aggregated_score, self.away_competitor.aggregated_score)


# ---------------------------------------------------------------------------------------------------------------------------------------- #